/**
 * This is an implementation of a Queue that supports
 * queuing and dequeuing from both ends.
 * @see QueueReferenceBased for other available methods.
 *
 * @author Zach Tomaszewski
 * @date27 Oct 2001
 */

public class DequeArrayBased extends QueueArrayBased
                              implements DequeInterface{

/**
 * Enqueues newItem on the front of the queue (similar to a push)
 * @param newItem - the object to be added to the front of the queue
 * @throws QueueException only if necessitated by implementation specific limits
 */
public void enqueueFront(Object newItem) throws QueueException{
  if (!isFull()) {
    front = (front-1 + MAX_QUEUE) % (MAX_QUEUE);
    items[front] = newItem;
    ++count;
  }else {
    throw new QueueException("QueueException on enqueueFront: "
                           + "Queue full");
  }
}


/**
 * Removes and returns the last item on a queue (similar to pop)
 * This makes the list one item shorter.
 * @return - the Object that was on the end of the queue
 * @throws QueueException if the queue is empty
 */
public Object dequeueBack() throws QueueException{
  if (!isEmpty()) {
    // queue is not empty; remove front
    Object queueBack = items[back];
    items[back] = null;
    back = (back - 1 + MAX_QUEUE) % (MAX_QUEUE);
    --count;
    return queueBack;
  }else {
    throw new QueueException("QueueException on dequeueBack: "+
    "Queue empty");
  }
}

}//end class

