/**
 * DequeTest.java 
 * Tests array-based and reference-based implementations of
 * enqueueFront and dequeueBack of DequeInterface (doubly ended
 * queue). enqueues and dequeues a string in such a manner that it
 * should come out in the same order or reversed. Also tests that an
 * exception is thrown when trying to dequeue an empty queue.
 * @author: Dan Suthers and Xiaohua Chen 
 * @version: 1.0 
 */

public class DequeTest {

    public static void main (String arg[]) {

        String data =  "Two can keep secret, if one of them is dead.";
    
        DequeInterface deque_array, deque_ref;
        deque_array = new DequeArrayBased();
        deque_ref = new DequeReferenceBased();

        // --- preserving order --- 
        System.out.println("enqueueFront and enqueueBack (should be in same order):");
        System.out.println("  DequeArrayBased:"); 
        System.out.println("    Original: " + data); 
        enqueueFrontString(data, deque_array); 
        System.out.print("    Result:   "); 
        dequeueBackAndPrint(deque_array); 
        System.out.println(); 

        System.out.println("  DequeReferenceBased:"); 
        System.out.println("    Original: " + data); 
        enqueueFrontString(data, deque_ref); 
        System.out.print("    Result:   "); 
        dequeueBackAndPrint(deque_ref); 
        System.out.println("\n"); 

        deque_array.dequeueAll();
        deque_ref.dequeueAll();


        // --- reversing order --- 
        System.out.println("enqueueFront and dequeue (should be in reverse order):");
        System.out.println("  DequeArrayBased:"); 
        System.out.println("    Original: " + data); 
        enqueueFrontString(data, deque_array); 
        System.out.print("    Result:   "); 
        dequeueAndPrint(deque_array); 
        System.out.println(); 

        System.out.println("  DequeReferenceBased:"); 
        System.out.println("    Original: " + data); 
        enqueueFrontString(data, deque_ref); 
        System.out.print("    Result:   "); 
        dequeueAndPrint(deque_ref); 
        System.out.println("\n"); 

        deque_array.dequeueAll();
        deque_ref.dequeueAll();

        // Test for throwing exception 
        System.out.println("dequeueBack of emtpy queue (should print exception)");
        System.out.println("  DequeArrayBased: "); 
        System.out.print("    "); 
        try {
            System.out.println(deque_array.dequeueBack()); 
        }
        catch (QueueException e) {
            System.out.println(e); 
        }
        System.out.println("  DequeReferenceBased:"); 
        System.out.print("    "); 
        try {
            System.out.println(deque_ref.dequeueBack()); 
        }
        catch (QueueException e) {
            System.out.println(e); 
        }
    }

    public static void enqueueString(String data, DequeInterface deque) { 
        for (int i = 0; i < data.length(); i++) {
            Character ch = new Character(data.charAt(i));
            deque.enqueue(ch);
        }
    }


    public static void enqueueFrontString(String data, DequeInterface deque) { 
        for (int i = 0; i < data.length(); i++) {
            Character ch = new Character(data.charAt(i));
            deque.enqueueFront(ch);
        }
    }


    public static void dequeueAndPrint(DequeInterface deque) { 
        while(!deque.isEmpty()) {
            Character ch = (Character)deque.dequeue();
            System.out.print(ch);
        }
    }

    public static void dequeueBackAndPrint(DequeInterface deque) { 
        while(!deque.isEmpty()) {
            Character ch = (Character)deque.dequeueBack();
            System.out.print(ch);
        }
    }


}

