
/**
 * A utility class of nodes to be used in implementing linked lists
 * with links going in both directions.  Not that, because this class
 * extends Node, all methods return Nodes.  Casting is often necessary.
 *
 * @author Zach Tomaszewski
 * @date 29 Sept 2001
 * @see EditableTuringTape
 */
class DoublyLinkedNode extends Node {
   private Node prevNode; 

 protected DoublyLinkedNode(Object item, Node prev, Node nextNode) {
   super(item, nextNode);
   this.prevNode = prev;
 }

 protected DoublyLinkedNode(Object item){
   this(item, null, null);
 }



 /**
  * gets the previous node
  */
 protected Node getPrev(){
   return this.prevNode;
 }


 /**
  * updates the link from this node to the previous one
  */
 protected void setPrev(Node prevNode){
   this.prevNode = prevNode;
 }

 /**
  * removes the link from this node to the previous one (sets it to null) 
  */
 protected void removePrev() {
    this.prevNode = null;
 }
}

